
package com.textura.cpms.web.servlet;

import javax.inject.Inject;
import org.glassfish.osgicdi.OSGiService;

import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import java.io.*;
import java.util.ResourceBundle;
import java.util.Locale;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.*;

import com.textura.cpms.service.hello.HelloService;

@WebServlet(urlPatterns="/*")
public class SayHello extends HttpServlet {

	private static class HelloServiceLocator {
		private final ServiceTracker serviceTracker;

		public HelloServiceLocator(BundleContext context) {
			serviceTracker = new ServiceTracker(context, HelloService.class.getName(), null);
			serviceTracker.open();
		}
		
		public HelloService getHelloService(long timeout) {		
			HelloService service = null;
			try {
				service = HelloService.class.cast(timeout == -1 ? serviceTracker.getService() : serviceTracker.waitForService(timeout));
			} catch (InterruptedException e) {
				throw new RuntimeException(e);
			}
			if (service == null) throw new RuntimeException("Unable to locate HelloService");
			return service;
		}

		public void close() {
			serviceTracker.close();
		}
	}	
	
	@Override
	public void init() {
		helloServiceLocator = new HelloServiceLocator(getBundleContext());
		helloService = helloServiceLocator.getHelloService(0);
	}
	
	@Override
	public void destroy() {
		if (helloServiceLocator != null) {
			helloServiceLocator.close();
		}
	}	

	private HelloServiceLocator helloServiceLocator;
	
//	@Inject 
//	@OSGiService(dynamic = true) // Glassfish specific extension :(
	private HelloService helloService;
	

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		PrintWriter writer = response.getWriter();
		Locale locale = request.getLocale();
		System.out.println("request locale = " + locale);
		String bundleName = "cpms.resources.i18n.messages";
		ResourceBundle resources = ResourceBundle.getBundle(bundleName, locale);
		String greeting = resources.getString("SayHello.hello");
		writer.append(greeting);		
		writer.append(helloService.sayHello());
		writer.close();
	}
	
    private BundleContext getBundleContext() {
        return BundleContext.class.cast(getServletConfig().getServletContext().getAttribute("osgi-bundlecontext"));
    }	
}
